import java.awt.Color;
import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

/**
 * This class constructs a window in which the results of a search are displayed
 * 
 * @author Sandino Vargas-Perez
 * @version 31 MAy 2020
 *
 */
public class Results implements ActionListener {
	Engine engine;
	JFrame frame;

	public Results(Engine eng) {

		engine = eng;
		frame = null;
	}

	/**
	 * Displays the synonyms and antonyms of a word in a window panel.
	 * 
	 * @param results a DictionaryEntry object containing the results
	 */
	public void displayResults(DictionaryEntry results) {

		// Adds a title to the window
		String className = engine.getClass().getName();
		frame = new JFrame(className + " Results");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(600, 600);

		// Set the position in the display where the window will appear
		if (className.equals("SearchEngine2")) {
			frame.setLocation(700, 0);
		}

		Container container = frame.getContentPane();
		container.setLayout(new GridLayout(4, 1));

		JPanel p1 = new JPanel();
		p1.setSize(100, 100);
		JLabel msg = new JLabel("<html><font size=\"7\"><b>\"" + results.getWord() + "\"</b></font></html>");
		p1.add(msg);
		container.add(p1);

		// Builds a panel to display the syns. and ants. of the word
		displayNyms(results.getSynonymsAsArray(), "Synonyms", container);
		displayNyms(results.getAntonymsAsArray(), "Antonyms", container);

		// Adds a button to search more
		JPanel p4 = new JPanel();
		JButton btn = new JButton(makeLink("Search another word"));
		btn.addActionListener(this);
		p4.add(btn);
		container.add(p4);

		// Makes the wondows appear
		frame.setVisible(true);
	}

	/**
	 * Displays the list of synonyms or antonyms.
	 * 
	 * @param list      array of syns. or ants.
	 * @param label     title for the panel.
	 * @param container the container of this panels.
	 */
	private void displayNyms(String[] list, String label, Container container) {

		JPanel p = new JPanel();
		p.setBorder(BorderFactory.createTitledBorder("<html><b>" + label + "</b></html>"));

		if (list != null) {
			for (String w : list) {
				JButton button = new JButton();

				// Generate a "link"-like button if the particular word exists
				// in the engine
				if (engine.contains(w))
					button.setText(makeLink(w));
				else
					button.setText(w);

				button.setHorizontalAlignment(SwingConstants.LEFT);
				button.setBorderPainted(false);
				button.setOpaque(false);
				button.setBackground(Color.WHITE);
				button.setToolTipText(w);
				button.addActionListener(this);
				p.add(button);

			}

		} else {
			JLabel msg = new JLabel(label + " not found.");
			p.add(msg);
		}

		container.add(p);

	}

	/**
	 * Display a "No Found" message if word was not found
	 * 
	 * @param word the word searched for
	 */
	public void displayNotFound(String word) {
		String className = engine.getClass().getName();

		frame = new JFrame("Results");
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frame.setSize(600, 600);

		if (className.equals("SearchEngine2")) {
			frame.setLocation(700, 0);
		}
		Container container = frame.getContentPane();
		container.setLayout(new GridLayout(2, 1));
		JPanel p = new JPanel();
		p.setBorder(BorderFactory.createTitledBorder("<html><b> Not Found </b></html>"));

		JLabel msg = new JLabel(
				"<html><font size=\"6\" color=\"#FF0000\"><b>\"" + word + "\"</b> was not found.</font></htmnl>");
		p.add(msg);

		container.add(p);
		JPanel p2 = new JPanel();
		JButton btn = new JButton(makeLink("Search another word"));
		btn.addActionListener(this);
		p2.add(btn);
		container.add(p2);
		frame.setVisible(true);

	}

	/**
	 * Makes a "link" like label using HTML
	 * 
	 * @param word the word to be inserted in the "link"
	 * @return a "link" like string label
	 */
	private String makeLink(String word) {
		return "<html><font color=\"#0000FF\"><u>" + word + "</u></font></html>";
	}

	/**
	 * Returns the label without the HTML tags
	 * 
	 * @param link the link to be parsed
	 * @return a label
	 */
	private String getLabelFromLink(String link) {
		if (link.charAt(0) == '<')
			return link.substring(link.indexOf("<u>") + 3, link.indexOf("</u>"));
		else
			return null;
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String word = getLabelFromLink(((JButton) e.getSource()).getText());
		if (word != null)
			open(word);
	}

	/**
	 * When a word is clicked on, this method is executed. It calls the engine for a
	 * new search with the word clicked as parameter.
	 * 
	 * @param word to be searched for
	 */
	private void open(String word) {
		frame.setVisible(false);
		if (word.equals("Search another word"))
			word = ValidatedInputReader.getString("Search for word", "");
		engine.search(word);
	}
}
